"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ts_simple_ast_1 = require("ts-simple-ast");
const angular_external_resources_analyzer_1 = require("../external-resources/angular-external-resources.analyzer");
class AngularExternalStylesAnalyzer extends angular_external_resources_analyzer_1.AngularExternalResourcesAnalyzer {
    static getExternalStyles(sourceFile) {
        return sourceFile
            .getClasses()
            .reduce((externalStylesAssignment, classDeclaration) => {
            try {
                return [
                    ...externalStylesAssignment,
                    this.getComponentDecoratorPropertyOrThrow(classDeclaration, 'styleUrls')
                ];
            }
            catch (_a) {
                return externalStylesAssignment;
            }
        }, [])
            .map((externalStylesAssignment) => {
            const externalStylesKey = this.getExternalStylesAssignmentKeyOrThrow(externalStylesAssignment);
            const externalStyleValues = this.getExternalStylesAssignmentValuesOrThrow(externalStylesAssignment);
            const externalStyles = externalStyleValues
                .map((externalStyleValue) => {
                const externalStylePath = this.getExternalResourcePath(externalStyleValue);
                const resolvedExternalStylePath = this.resolveExternalResourcePath(externalStylePath, sourceFile);
                return {
                    node: externalStyleValue,
                    path: resolvedExternalStylePath
                };
            });
            return {
                node: externalStylesKey,
                styles: externalStyles
            };
        });
    }
    static getExternalStylesAssignmentKeyOrThrow(externalStylesAssignment) {
        return externalStylesAssignment.getChildrenOfKind(ts_simple_ast_1.SyntaxKind.Identifier)[0] ||
            (() => { throw new Error('External template does not have a key.'); })();
    }
    static getExternalStylesAssignmentValuesOrThrow(externalStylesAssignment) {
        const externalStylesAssignmentValue = externalStylesAssignment.getChildrenOfKind(ts_simple_ast_1.SyntaxKind.ArrayLiteralExpression)[0] ||
            (() => { throw new Error('External template doesn ot have a vlaue.'); })();
        const externalStylesAssignmentValues = externalStylesAssignmentValue.getElements();
        return externalStylesAssignmentValues;
    }
}
exports.AngularExternalStylesAnalyzer = AngularExternalStylesAnalyzer;
//# sourceMappingURL=angular-external-styles.analyzer.js.map