"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ts_simple_ast_1 = require("ts-simple-ast");
const angular_external_resources_analyzer_1 = require("../external-resources/angular-external-resources.analyzer");
class AngularExternalTemplatesFileAnalyzer extends angular_external_resources_analyzer_1.AngularExternalResourcesAnalyzer {
    static getExternalTemplates(sourceFile) {
        return sourceFile
            .getClasses()
            .reduce((externalTemplateAssignment, classDeclaration) => {
            try {
                return [
                    ...externalTemplateAssignment,
                    this.getComponentDecoratorPropertyOrThrow(classDeclaration, 'templateUrl')
                ];
            }
            catch (_a) {
                return externalTemplateAssignment;
            }
        }, [])
            .map((externalTemplateAssignment) => {
            const externalTemplateKey = this.getExternalTemplateAssignmentKeyOrThrow(externalTemplateAssignment);
            const externalTemplateValue = this.getExternalTemplateAssignmentValueOrThrow(externalTemplateAssignment);
            const externalTemplatePath = this.getExternalResourcePath(externalTemplateValue);
            const resolvedExternalTemplatePath = this.resolveExternalResourcePath(externalTemplatePath, sourceFile);
            return {
                node: externalTemplateKey,
                template: {
                    node: externalTemplateValue,
                    path: resolvedExternalTemplatePath
                }
            };
        });
    }
    static getExternalTemplateAssignmentKeyOrThrow(externalTemplateAssignment) {
        return externalTemplateAssignment.getChildrenOfKind(ts_simple_ast_1.SyntaxKind.Identifier)[0] ||
            (() => { throw new Error('External template does not have a key.'); })();
    }
    static getExternalTemplateAssignmentValueOrThrow(externalTemplateAssignment) {
        return externalTemplateAssignment.getChildrenOfKind(ts_simple_ast_1.SyntaxKind.StringLiteral)[0] ||
            (() => { throw new Error('External template doesn ot have a vlaue.'); })();
    }
}
exports.AngularExternalTemplatesFileAnalyzer = AngularExternalTemplatesFileAnalyzer;
//# sourceMappingURL=angular-external-templates.analyzer.js.map