"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const get_file_type_1 = require("./get-file-type");
function readFile(filePath) {
    return new Promise((resolve, reject) => {
        fs.readFile(filePath, 'utf-8', (readFileError, fileContent) => {
            if (readFileError) {
                reject(new Error(`An error occured while reading the file "${filePath}". [Code "${readFileError.code}", Number "${readFileError.errno}"]`));
                return;
            }
            let parsedFileContent = fileContent;
            if (get_file_type_1.getFileType(filePath) === 'json') {
                try {
                    parsedFileContent = JSON.parse(fileContent);
                }
                catch (jsonParseError) {
                    reject(new Error(`An error occured while parsing the file "${filePath}" as JSON. [${jsonParseError.message}]`));
                    return;
                }
            }
            resolve(parsedFileContent);
        });
    });
}
exports.readFile = readFile;
//# sourceMappingURL=read-file.js.map